# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging
import time

from driverng import Worklist
from driverng.astm import AstmMessageV2, HeaderSegment
from driverng.astm import HprimDate as HL7Date
from driverng.astm import HprimForeign as HL7Foreign
from driverng.astm import HprimList as HL7List
from driverng.astm import HprimString as HL7String
from driverng.astm import Segment

logger = logging.getLogger("driverng.elo_hl7")


class ELOHL7MSH(HeaderSegment):
    sending_application = HL7String(3)
    sending_facility = HL7String(4)
    date = HL7Date(7)
    message_code = HL7String((9, 1), default="ORM")
    trigger_event = HL7String((9, 2), default="001")
    control_id = HL7String(10)
    processing_id = HL7String(11, default="P")
    version_id = HL7String(12, default="2.4")
    country_code = HL7String(17, default="FRA")
    language = HL7String(19, default="FR")

    patients = HL7Foreign("PID", "header")


class ELOHL7PID(Segment):
    patient_id = HL7String(2)
    optional_id = HL7String(3)
    last_name = HL7String((6, 1))
    first_name = HL7String((6, 2))
    birthdate = HL7Date(8, fmt_date="%Y%m%d")
    sex = HL7String(9)

    common_orders = HL7Foreign("ORC", "patient")

    def to_string(self, separators=None):
        self.rang_segment = self.patient_id
        return super(ELOHL7PID, self).to_string(separators=separators)


class ELOHL7ORC(Segment):
    order_control = HL7String(2)
    placer_order_number = HL7String(3)
    transaction_date = HL7Date(10)
    ordering_provider_id = HL7String((13, 1))
    ordering_provider_name = HL7String((13, 2))
    ordering_provider_firstname = HL7String((13, 3))

    orders = HL7Foreign("OBR", "common_order")

    def to_string(self, separators=None):
        self.rang_segment = self.order_control
        return super(ELOHL7ORC, self).to_string(separators=separators)


class ELOHL7OBR(Segment):
    barcode = HL7String(3)
    priority = HL7String(6, default="R")
    observation_date = HL7Date(8)
    specimen_type = HL7String(16)

    results = HL7Foreign("OBX", "order")


class ELOHL7OBX(Segment):
    value_type = HL7String(3)
    analysis_identifier = HL7String((4, 1))
    analysis_text = HL7String((4, 2))
    value = HL7String(6)
    units = HL7String(7)
    reference_ranges = HL7String(8)
    abnormal_flags = HL7List(9)
    observation_result_status = HL7String(12)
    result_time = HL7Date(13)
    observation_date = HL7Date(15)


class Driver(AstmMessageV2):
    SPEC = {"code": "elo_hl7", "label": "EUROLabOffice HL7"}
    SEGMENT_CLASS_MAP = {
        "MSH": ELOHL7MSH,
        "PID": ELOHL7PID,
        "ORC": ELOHL7ORC,
        "OBR": ELOHL7OBR,
        "OBX": ELOHL7OBX,
    }
    DEFAULT_SEPARATORS = "|^~\\&"
    FIRST_SEP_INDEX = 3
    SEGMENT_ENDING = "\r"

    OPTIONS = [
        AstmMessageV2.Option("elo_hl7_qc_prefix", label="Préfixe QC"),
        AstmMessageV2.Option(
            "elo_hl7_sender_id",
            label="Paramétrer l'ID de l'émetteur (champ MSH.3)",
            default="MCA",
        ),
        AstmMessageV2.Option(
            "elo_hl7_sender_name",
            label="Paramétrer le nom de l'émetteur (champ MSH.4)",
            default="MCA",
        ),
    ]

    def __init__(self, **kwargs):
        self.elo_hl7_qc_prefix = kwargs.pop("elo_hl7_qc_prefix", "")
        self.elo_hl7_sender_id = kwargs.pop("elo_hl7_sender_id", "MCA")
        self.elo_hl7_sender_name = kwargs.pop("elo_hl7_sender_name", "MCA")
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        return ELOHL7MSH(type_segment="MSH", **kwargs)

    def to_string(self, message, separators=None, **kwargs):
        return super(Driver, self).to_string(message, separators, **kwargs) + "\r"

    @staticmethod
    def create_orc_segment(seg_pid, action, sample, prescriber):
        return seg_pid.common_orders.add(
            order_control=action,
            placer_order_number=sample.request_id,
            transaction_date=sample.request_date,
            ordering_provider_id=prescriber[0].code if prescriber else "",
            ordering_provider_name=prescriber[0].last_name if prescriber else "",
            ordering_provider_firstname=prescriber[0].first_name if prescriber else "",
        )

    @staticmethod
    def create_obr_segment(seg_orc, sample):
        return seg_orc.orders.add(
            barcode=sample.sample_id,
            observation_date=sample.request_date,
            specimen_type=sample.specimen,
        )

    def encode_worklist(self, worklist, context):
        message = self.create(
            sending_application=self.elo_hl7_sender_id,
            sending_facility=self.elo_hl7_sender_name,
            date=context["now"],
            control_id=context.get("control_id", int(time.time() * 1000)),
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["action"], action={"del": "CA", worklist.ELSE: "NW"}
        ):
            prescriber = [p for p in patient.physicians if p.type == "prescriber"]
            seg_pid = message.patients.add(
                patient_id=patient.patient_id,
                optional_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
            )
            for sample, action in patient.samples:
                # Order
                for order in sample.orders:
                    seg_orc = self.create_orc_segment(
                        seg_pid, action, sample, prescriber
                    )
                    seg_obr = self.create_obr_segment(seg_orc, sample)
                    seg_obr.results.add(
                        analysis_identifier=order.code,
                        analysis_text=order.label,
                        observation_result_status="I",
                        observation_date=sample.request_date,
                    )
                # External results
                for result in sample.results:
                    action = "XE"
                    seg_orc = self.create_orc_segment(
                        seg_pid, action, sample, prescriber
                    )
                    seg_obr = self.create_obr_segment(seg_orc, sample)
                    seg_obr.results.add(
                        value_type=result.type,
                        analysis_identifier=result.code,
                        analysis_text=result.label,
                        value=result.value,
                        units=result.unit,
                        reference_ranges=result.reference_ranges,
                        abnormal_flags=result.flags,
                        observation_result_status="P",
                        result_time=result.date,
                        observation_date=sample.request_date,
                    )

        if message.patients:
            return message

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_pid in message.patients:
            for seg_orc in seg_pid.common_orders:
                for seg_obr in seg_orc.orders:
                    if self.elo_hl7_qc_prefix and seg_obr.barcode.startswith(
                        self.elo_hl7_qc_prefix
                    ):
                        sample = worklist.get_or_create(
                            sample_id=seg_obr.barcode,
                            control_lot=seg_obr.barcode,
                            type="control",
                        )
                    else:
                        sample = worklist.get_or_create(
                            sample_id=seg_obr.barcode, type="patient"
                        )
                    for seg_obx in seg_obr.results:
                        result = sample.results.add(
                            code=seg_obx.analysis_identifier,
                            date=seg_obx.observation_date,
                            value=seg_obx.value,
                            flags=seg_obx.abnormal_flags,
                        )
                        if seg_obx.units:
                            result.unit = seg_obx.units
        return worklist
